#![feature(async_closure)]
#![feature(associated_type_bounds)]
#![allow(unused_parens)]
#[macro_use]
extern crate async_trait;
extern crate env_logger;
extern crate futures;
extern crate libunftp;
extern crate tokio;

use std::error::Error;
use std::sync::Arc;

pub(crate) mod auth;
mod error;
mod env;
mod storage;

#[tokio::main]
async fn main() -> Result<(), Box<dyn Error>> /* {{{ */ {
	let port = match env::get("YADMS_FTP_PORT") {
		Some(s) => s.parse::<u16>().expect("YADMS_FTP_PORT must be an integer between 1 and 65535"),
		None => 21
	};

	let min_passive_port = match env::get("YADMS_FTP_PASSIVE_PORT_MIN") {
		Some(s) => s.parse::<u16>().expect("YADMS_FTP_PASSIVE_PORT_MIN must be an integer between 1024 and 65535"),
		None => 21000
	};

	let max_passive_port = match env::get("YADMS_FTP_PASSIVE_PORT_MAX") {
		Some(s) => s.parse::<u16>().expect("YADMS_FTP_PASSIVE_PORT_MAX must be an integer between 1024 and 65535"),
		None => 21999
	};

	if(min_passive_port >= max_passive_port) {
		eprintln!("YADMS_FTP_PASSIVE_PORT_MAX must be greater than YADMS_FTP_PASSIVE_PORT_MIN");
	}

	if(min_passive_port < 1024 || max_passive_port < 1024) {
		eprintln!("YADMS_FTP_PASSIVE_PORT_MIN and YADMS_FTP_PASSIVE_PORT_MAX must be greater than 1023");
	}

	let authenticator = auth::Authenticator::new().await.expect("authenticator new fail");
	let generator = Box::new(|| futures::executor::block_on(storage::Backend::new()).expect("backend new fail"));

	std::env::set_var("RUST_LOG", "libunftp=info");
	env_logger::init();
	//let server = libunftp::Server::new_with_fs_root(env::get("YADMS_DOCUMENT_ROOT").unwrap())
	let server = libunftp::Server::with_authenticator(generator, Arc::new(authenticator))
		.greeting("Welcome to the YADMS FTP interface")
		.passive_ports(min_passive_port..(max_passive_port + 1));
	server.listen(format!("0.0.0.0:{}", port)).await.unwrap();
	Ok(())
} // }}}

