use std::error::Error;
use std::path::Path;

#[derive(Debug)]
pub struct InvalidPathError {
	path: std::path::PathBuf
}

impl std::fmt::Display for InvalidPathError {
	fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
		write!(f, "Path '{:?}' is not in either /documents or /tags", self.path.to_str())
	}
}

impl std::error::Error for InvalidPathError {
	fn description(&self) -> &str {
		"Path is not in either /documents or /tags"
	}

	fn source(&self) -> Option<&(dyn Error + 'static)> {
		None
	}
}

impl InvalidPathError {
	pub(crate) fn new(path: impl AsRef<Path>) -> Self {
		Self{path: path.as_ref().to_path_buf()}
	}
}

