extern crate libunftp;

use std::os::unix::fs::MetadataExt;
use std::time::SystemTime;

#[derive(Debug)]
pub(crate) struct Metadata {
	len: u64,
	is_dir: bool,
	is_file: bool,
	is_symlink: bool,
	mtime: SystemTime,
	gid: u32,
	uid: u32
}

impl Metadata {
	pub(crate) fn new(len: u64, is_dir: bool, is_file: bool, is_symlink: bool, mtime: SystemTime, gid: u32, uid: u32) -> Self {
		Self{
			len: len,
			is_dir: is_dir,
			is_file: is_file,
			is_symlink: is_symlink,
			mtime: mtime,
			gid: gid,
			uid: uid
		}
	}
}

impl From<std::fs::Metadata> for Metadata {
	fn from(other: std::fs::Metadata) -> Self {
		Self{
			len: other.len(),
			is_dir: other.is_dir(),
			is_file: other.is_file(),
			is_symlink: other.is_symlink(),
			mtime: other.modified().expect("mtime fail"),
			uid: other.uid(),
			gid: other.gid()
		}
	}
}

impl libunftp::storage::Metadata for Metadata {
	fn len(&self) -> u64 {
		self.len
	}

	fn is_dir(&self) -> bool {
		self.is_dir
	}

	fn is_file(&self) -> bool {
		self.is_file
	}

	fn is_symlink(&self) -> bool {
		self.is_symlink
	}

	fn modified(&self) -> Result<SystemTime, libunftp::storage::Error> {
		Ok(self.mtime)
	}

	fn uid(&self) -> u32 {
		self.uid
	}

	fn gid(&self) -> u32 {
		self.gid
	}
}

